clear all;
close all;

%% Get function handles
vsf = vision_system_functions; 

%% Constants

% This will produce several figures if set to true
debug = false;

% Red pixel search thresholds
T_1 = 1.3; T_2 = 0.71;

% Image size
V = 480; % Height    
U = 640; % Width

% Known real-world distance 
d_real = 35; %[mm]

% Camera
f_mm = 3.04; 
SENSOR_WIDTH_MM = 3.68;
s_c = U / SENSOR_WIDTH_MM; % Compute pixel per unit length
f_pix = s_c * f_mm; % Compute focal length in unit [pixel]

u_0 = round(U / 2); v_0 = round(V / 2); % Optical center

% Euler angles: CAM to BODY
Theta_b_to_n = deg2rad([0 0 0]'); % Angles FROM BODY-frame to NED-frame
R_c_to_b = vsf.zyx_rot(Theta_c_to_b);

% Euler angles: BODY to NED
Theta_c_to_b = deg2rad([0 0 0]'); % Angles FROM CAM-frame TO BODY-frame



%% Algorithm Example

I_rgb = imread('images\vertical_translation\image_lr_gt800_7.jpg');


% Normalize and threshold RGB image
I_rgb_norm = vsf.normalize_rgb_image(I_rgb);
I_bin = vsf.find_red_pixels_from_rgb(I_rgb_norm, T_1, T_2);

% Extract contours
raw_contours = bwboundaries(I_bin);
if debug
    vsf.show_contours(raw_contours, I_bin);
    th = title('Unfiltered Contours');
    th.FontSize = 25;
    waitfor(gcf);
end

% Filter contours
filtered_contours = vsf.countour_filter(raw_contours, U, V);
if debug
    vsf.show_contours(filtered_contours, I_bin);
    th = title('Filtered Contours');
    th.FontSize = 25;
    waitfor(gcf);
end

% Extract feature points from filtered contours
[u_min, u_max, v_min, v_max] = vsf.extract_feature_points_from_countours(filtered_contours);

% Compute discrete target coordinates based on feature points
[u_target, v_target] = vsf.find_target(u_min, u_max, v_min, v_max);

if debug
    vsf.plot_feature_points(u_min, u_max, v_min, v_max, u_target, v_target, u_0, v_0, I_bin)
    th = title('Feature Points');
    th.FontSize = 25;
    waitfor(gcf);
end

% Compute position in CAM-frame [mm]
P_c = vsf.estimate_cam_position(f_pix, u_min, u_max, v_min, v_max, u_0, v_0, u_target, v_target, d_real);

% Compute rotation
R_b_to_n = vsf.zyx_rot(Theta_b_to_n);

% Compute relative position
P_n_uav_rel = - R_b_to_n * (R_c_to_b * P_c);



